<?php

error_reporting(E_ALL);  // Let's show all errors


define("MAME_PATH", "C:/GP32/MAME/mame19_mod/");

$GLOBALS['drivers'] = array();
$GLOBALS['fxelist'] = array();
$GLOBALS['aliases'] = array();

parse_aliases();

parse_makefiles();
parse_driver_hs();
parse_driver_cs();
parse_extras();

validate_drivers();
write_drivers();

//print_r($GLOBALS['drivers']);


print "done";
exit;



function validate_drivers()
{
  global $drivers;

  foreach($drivers as $GameDriver => $Record)
  {
    if(!isset($Record->Genre))
    {
      if($Record->CloneOf <> "")
      {
        $Key = strtolower($Record->CloneOf);
        $Key .= "_driver";
        
        if(isset($drivers[$Key]->Genre))
          $Record->Genre = $drivers[$Key]->Genre;
        if(isset($drivers[$Key]->Category))
          $Record->Category = $drivers[$Key]->Category;
      }
    }
  }

  // Validate everything was found
  foreach($drivers as $GameDriver => $Record)
  {
    if(!isset($Record->Manufacturer))
      die("Info was not found for $GameDriver");
      
    if(!isset($Record->Genre) || $Record->Genre == "")
      print "$GameDriver : No Genre  CloneOf: $Record->CloneOf\n";

    if(!isset($Record->Category) || $Record->Category == "")
      print "$GameDriver : No Category  CloneOf: $Record->CloneOf\n";
  }

  //print_r($drivers);
}


function write_drivers()
{
  global $drivers;
  
  usort($drivers, "cmp");
  $gamecount = count($drivers);
  
  
  $fp = fopen("drivers.h", "wb");
  fprintf($fp, 
  "#ifndef _DRIVERS_H_
  #define _DRIVERS_H_
  
  typedef struct DRIVER
  {
    char *description;
    char *name;
    char *fxe;
    char year[5];
    char *manufacturer;
    char *cloneof;
    char *genre;
    char *category;
    int available;
  } DRIVER;\n\n
  
  #define NUMGAMES $gamecount
  extern DRIVER drivers[NUMGAMES];
  extern char *AutoGenreFilter;
  extern char *AutoCategoryFilter;
  extern char *AutoFXEFilter;
  extern char *AutoYearFilter;
  extern char *AutoCloneofFilter;
  extern char *AutoManufacturerFilter;
  
  #endif\n\n");
  fclose($fp);
  
  
  $fp = fopen("drivers.c", "wb");
  fprintf($fp, "#include \"drivers.h\"
DRIVER drivers[NUMGAMES] = 
{\n");
  
  foreach($drivers as $key => $rec)
  {
    fprintf($fp, "  { \"$rec->Description\", \"$rec->Name\", \"$rec->FXE\", \"$rec->Year\", \"$rec->Manufacturer\", \"$rec->CloneOf\", \"$rec->Genre\", \"$rec->Category\", 0 } ,\n");
  //  fprintf($fp, "  { \"$rec->Description\", \"$rec->Name\", \"$rec->FXE\", 0 } ,\n");
  }
  fprintf($fp, "};\n\n\n");
  
  WriteAutoScript($fp, 'Genre', 'BY GENRE', 'AutoGenreFilter');
  WriteAutoScript($fp, 'Category', 'BY CATEGORY', 'AutoCategoryFilter');
  WriteAutoScript($fp, 'FXE', 'BY FXE', 'AutoFXEFilter');
  WriteAutoScript($fp, 'Year', 'BY YEAR', 'AutoYearFilter');
  WriteAutoScript($fp, 'CloneOf', 'BY CLONEOF', 'AutoCloneofFilter');
  WriteAutoManufacturerScript($fp);
  
  fclose($fp);
}

function WriteAutoScript($fp, $field, $bystr, $variname)
{
  global $drivers;
  
  $list = array();
  foreach($drivers as $driver => $record)
    $list[$record->$field] = 0;
    
  ksort($list, SORT_STRING);

  fprintf($fp, "char *$variname = \"Filter-Group=\\\"$bystr\\\" { \" \n" );
  foreach($list as $key => $record)
    fprintf($fp, "                        \"  \\\"$key\\\" $field equals (\\\"$key\\\") \"\n");
  fprintf($fp, "                        \"}\";\n\n\n\n");
}   


function WriteAutoManufacturerScript($fp)
{
  global $drivers;
  
  $list = array();
  foreach($drivers as $driver => $record)
  {
    if(!isset($list[$record->Manufacturer]))
      $list[$record->Manufacturer] = 0;
    else
      $list[$record->Manufacturer]++;
  }
    
  ksort($list, SORT_STRING);

  fprintf($fp, "char *AutoManufacturerFilter = \"Filter-Group=\\\"By Manufactuer\\\" { \" \n" );
  foreach($list as $key => $count)
  {
    if($count > 2)
      fprintf($fp, "                        \"  \\\"$key\\\" MANUFACTURER equals (\\\"$key\\\") \"\n");
  }
  fprintf($fp, "                        \"  \\\"Various\\\" MANUFACTURER equals (");

  foreach($list as $man => $count)
  {
    if($count <= 2)
      fprintf($fp, "\\\"$man\\\", ");
  }
  fprintf($fp, "\\\"0\\\") \" \n");
  
  fprintf($fp, "                        \"}\";\n\n\n\n");
}   




function cmp($a, $b) 
{   
  if ($a->Description == $b->Description) 
    return 0;
  return ($a->Description > $b->Description) ? 1 : -1;
}


function parse_driver_hs()
{
  $files = scandir(MAME_PATH . 'src');
  foreach($files as $key => $file)
  {
    $file = strtolower($file);
    if($file != 'driver_all.c' && substr($file, 0, 7) == 'driver_')
      parse_driver_h(MAME_PATH . "src/$file");
  }
}

function parse_driver_h($file)
{
  global $drivers;
  global $fxelist;

  $CurPos = 0;  
  if(!preg_match('/(driver_\w+)/smi', $file, $var, PREG_OFFSET_CAPTURE, $CurPos)) die("unable to parse $file");
  $driver = $var[1][0];

  foreach($fxelist as $check_fxe => $check_driver)
  {
    if(strcasecmp($check_driver, $driver) == 0)
      $fxe = $check_fxe;
  }
  if(!isset($fxe)) die("Unable to find fxe driver for $file $driver");


  $fp = fopen($file, "r");
  if($fp === false)
    die("unable to open $file");

  while(!feof($fp))
  {
    $line = fgets($fp);
    if($line === false)
      break;

    $sub = substr($line, 0, 24);
    if($sub == 'extern struct GameDriver')
    {
      $sub = substr($line, 25);
      $x = strpos($sub, ';');
      if($x !== false)
        $sub = substr($sub, 0, $x);

      $sub = strtolower($sub); 

      $drivers[$sub]->GameDriver = $sub;
//      $drivers[$sub]->Driver = $driver;   /* c file driver decl was extacted from */
      $drivers[$sub]->FXE = $fxe;
    }
  }  
  fclose($fp);
}


function parse_driver_cs()
{
  $files = scandir(MAME_PATH . 'src/drivers');
  foreach($files as $key => $file)
  {
  
    if($file != '.' && $file != '..')
      parse_driver_c(MAME_PATH . "src/drivers/$file");
  }
}


function parse_driver_c($file)
{
  global $drivers;

  if($file == '.' || $file == '..') return;

  $contents = file_get_contents($file);
  if($contents === false) die("Unable to read $file");
    
  $CurPos = 0;
  while(1)
  {
    if(!preg_match("/struct\s+GameDriver\s+(\w+)\s*=/smi", $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $GameDriver = $var[1][0];
    $CurPos = $var[1][1] + 0;

    // Skip __FILE__
    if(!preg_match('/(\w+)\s*(,)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $CurPos = $var[2][1];

    // CloneOf
    if(!preg_match('/(\w+)\s*(,)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $CloneOf = $var[1][0];
    $x = strpos($CloneOf, '_');  // remove _driver 
    $CloneOf = substr($CloneOf, 0, $x);   
    if($CloneOf == '0') $CloneOf = '';
    $CurPos = $var[2][1] + 0;

    // Name
    if(!preg_match('/"([^"](?:\\.|[^\\"]*)*)"(,)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $Name = $var[1][0];
    $CurPos = $var[2][1] + 0;

    // Description
    if(!preg_match('/"([^"](?:\\.|[^\\"]*)*)"(,)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $Description = $var[1][0];
    $CurPos = $var[2][1] + 0;

    // Year
    if(!preg_match('/"([^"](?:\\.|[^\\"]*)*)"(,)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $Year = $var[1][0];
    $Year = substr($Year, 0, 4);
    if(!is_numeric($Year)) $Year = 'Unkn';  // unknown
    $CurPos = $var[2][1] + 0;

    // Manufacture
    if(!preg_match('/"([^"](?:\\.|[^\\"]*)*)"(,)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $Manufacturer = FixManufacturer($var[1][0]);
    $CurPos = $var[2][1] + 1;

    // Skip Credits
    $x = strpos($contents, ",", $CurPos);
    if($x !== false) $CurPos = $x + 1;

//print "Pos After Credits : " . substr($contents, $CurPos, 30) . "\n";

    // Flags (look for GAME_NOT_WORKING)
    if(!preg_match('/([\w\d]+)\s*(,)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) break;
    $Flags = $var[1][0];
    $CurPos = $var[2][1];

    if(strcasecmp($Flags, "GAME_NOT_WORKING") == 0)
    {
	    if(isset($drivers[$GameDriver]))
        unset($drivers[$GameDriver]);
      continue;
		}

    if(isset($drivers[$GameDriver]))
    {
      $drivers[$GameDriver]->CloneOf = $CloneOf;
      $drivers[$GameDriver]->Name = $Name;
      $drivers[$GameDriver]->Description = $Description;
      $drivers[$GameDriver]->Year = $Year;
      $drivers[$GameDriver]->Manufacturer = $Manufacturer;
    }
  }
}

function FixManufacturer($Manufacturer)
{
  if($Manufacturer == '????' || $Manufacturer == '?????' || $Manufacturer == '<unknown>' || $Manufacturer == 'bootleg' || $Manufacturer == 'bootleg?' || $Manufacturer == 'hack') $Manufacturer = "unknown";
  if($x = strpos($Manufacturer, '(')) $Manufacturer = trim(substr($Manufacturer, 0, $x));
  if($Manufacturer == '[Namco]') $Manufacturer = 'Namco';
  if($Manufacturer == '[SNK]') $Manufacturer = 'SNK';
  if($Manufacturer == '[Stern]') $Manufacturer = 'Stern';
  if($x = strpos($Manufacturer, '[')) $Manufacturer = trim(substr($Manufacturer, 0, $x));

  if($Manufacturer == 'Amenip/Centuri') $Manufacturer = "Centuri";
  if($Manufacturer == 'Atari + Gencomp') $Manufacturer = "Atari";
  if($Manufacturer == 'Bally Midway') $Manufacturer = 'Midway';
  if($Manufacturer == 'Coreland / Sega') $Manufacturer = "Sega / Coreland";
  if(substr($Manufacturer, 0, 9) == 'Data East') $Manufacturer = 'Data East';
  if($Manufacturer == 'GDI + Irem') $Manufacturer = 'Irem';
  if($Manufacturer == 'Game Plan, Inc.') $Manufacturer = 'GamePlan';
  if($Manufacturer == 'Konami + Kosuka') $Manufacturer = 'Konami';
  if($Manufacturer == 'Meadows Games, Inc.') $Manufacturer = 'Meadows';
  if($Manufacturer == 'Sigma Ent. Inc.') $Manufacturer = 'Sigma';
  if(strpos($Manufacturer, 'Taito') !== false)  $Manufacturer = 'Taito';
  if($Manufacturer == 'Technos + California Dreams') $Manufacturer = 'Technos';
  if($Manufacturer == 'Universal USA') $Manufacturer = 'Universal';
  if($Manufacturer == 'Zilec Games') $Manufacturer = 'Zilec-Zenitone';
  if($Manufacturer == 'Nintendo of America') $Manufacturer = 'Nintendo';


  return($Manufacturer);
}


function parse_makefiles()
{
  $files = scandir(MAME_PATH);
  foreach($files as $key => $file)
  {
    $file = strtolower($file);
    if(substr($file, 0, 9) == "makefile_")
      parse_makefile(MAME_PATH . $file);
  }

//print_r($GLOBALS['fxelist']);
//exit;
}

function parse_makefile($file)
{
  global $fxelist;
  $contents = file_get_contents($file);
  
  $CurPos = 0;

  if(!preg_match('/(\w+)\.fxe/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) die("Unable to find .fxe in $file");
  $fxe = $var[1][0];

  if(!preg_match('/(driver_\w+)/smi', $contents, $var, PREG_OFFSET_CAPTURE, $CurPos)) 
    return;  // must not be a mame makefile

  $driver = $var[1][0];
  $fxelist[$fxe] = $driver;
}


function parse_aliases()
{
  $fp = fopen("alias.ini", "r");
  if($fp === false) die("unable to open alias.ini");
  while(!feof($fp))
  {
    $line = fgets($fp);
    if($line === false) break;
    $line = strtolower($line);
    
    $line = str_replace(array("\r", "\n"), "", $line);

    $x = strpos($line, '=');
    if($x !== false)
    {
      $game1 = substr($line, 0, $x);
      $game2 = substr($line, $x+1);
      $GLOBALS['aliases'][$game1] = $game2;
    }
  }
  fclose($fp);
  
}


function parse_extras()
{
  parse_extra("Genre", "genre.ini");
  parse_extra("Category", "catlist.ini");
}


function parse_extra($field, $file)
{
  global $drivers, $aliases;

  $fp = fopen($file, "r");
  if($fp === false) die("unable to open $file");

  $CurSection = false;

  while(!feof($fp))
  {
    $line = fgets($fp);
    if($line === false) break;

    if(preg_match('/\\[(.*)\\]/', $line, $NewSection))
    {
//      print "Section = " . $NewSection[1] . "\n";
      if(strcasecmp($NewSection[1], "FOLDER_SETTINGS") == 0) 
        $CurSection = false;
      else
      {
        $CurSection = $NewSection[1];
        $CurSection = str_replace('(', '[', $CurSection);
        $CurSection = str_replace(')', ']', $CurSection);
        $CurSection = str_replace('"', "'", $CurSection);
        $CurSection = str_replace('.', "", $CurSection);

//        print "CurSection now equals $CurSection\n";
      }
    }      
    else if($CurSection !== false)
    {
      $line = trim($line);
      if($line <> '')
      {
        $driver = $line . "_driver";
        
        if(isset($aliases[$line]))
          $driver = sprintf("%s%s", ($aliases[$line]), "_driver");
     
        if(isset($drivers[$driver]))
          $drivers[$driver]->$field = $CurSection;
      }
    }
  }
  fclose($fp);
}

?>