#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

#include "util.h"
#include "filter.h"
#include "video.h"
#include "regexp.h"
#include "missinggames.h"
#include "drivers.h"

#ifdef __WIN32__
  int GP32_NEW_BLU=0;
#endif


void InitalizeConfig(void);
void parse_config(char *config);
int get_operatorcode(char *config, int *pos);
int get_fieldcode(char *config, int *pos);
void get_token(char *config, int *pos, char *ident);
void skip_line(char *config, int *pos);
void skip_white(char *config, int *pos);
int GetIntValue(char *config, int *pos);
int GetBooleanValue(char *config, int *pos);
char *get_value(char *config, int *pos, char *dest);
void read_filters(FILTER_GROUP *fp, char *config, int *pos);
void add_filter(FILTER_GROUP *fg, FILTER *filter);
FILTER_GROUP * add_filter_group(char *name);
void free_filters_groups(void);
void dump_filter_groups(void);
int opstring_to_code(char *opcode);
int fieldstring_to_code(char *str);
void parse_filter_values(FILTER *filter, char *buffer);
void ReadDriverLocations(char *config, int *pos);
void AddDriverLocation(DRIVERLOC *DriverLoc);
void ReadGameOptimizedDrivers(char *config, int *pos);
void AddGameOptimizedDriver(DRIVERLOC *DriverLoc);
void SetDriverLocation(char *LocDesc);

static int initialized = 0;
CONFIG_OPTIONS Config;

void ReadConfigFile(void)
{
  FILE *fp;
  int size;
  char *buffer = NULL;

  InitalizeConfig();
  	
	if(Config.GroupsAllocated)
	  free_filters_groups();
	  
  parse_config("Filter-Group=Unfiltered { All Desc REGEXP (.*) }");
	  
  fp = fopen(CFG_DIR "frontend.cfg", "rb");
	if(!fp)
		pause("%s not found", CFG_DIR "frontend.cfg");
	else
	{
	  fseek(fp, 0, SEEK_END);
	  size = ftell(fp);
	  fseek(fp, 0, SEEK_SET);
	  
	  buffer = (char *)malloc(size+10); 
	  fread(buffer, 1, size, fp);
	  fclose(fp);
	  buffer[size] = 0;
	  
	  parse_config(buffer);
	  free(buffer);
	}

  if(Config.AutoFXEFilter)  
    parse_config(AutoFXEFilter);

  if(Config.AutoYearFilter)
    parse_config(AutoYearFilter);

  if(Config.AutoCloneOfFilter)
    parse_config(AutoCloneofFilter);

  if(Config.AutoGenreFilter)
    parse_config(AutoGenreFilter);

  if(Config.AutoCategoryFilter)
    parse_config(AutoCategoryFilter);  

  if(Config.AutoManufacturerFilter)
    parse_config(AutoManufacturerFilter);  


  /* If no driver locations where specified in config, create a default */    
  if(Config.DriverLocationsCount == 0)
  {
  	DRIVERLOC DL = {"standard", "/gpmm/mamegp32"};
  	AddDriverLocation(&DL);
  }
  
#if 0  
	pause("Driver Locations : %d", Config.DriverLocationsCount);
	for(x = 0; x < Config.DriverLocationsCount; x++)
		pause("%s %s", Config.DriverLocations[x].Desc, Config.DriverLocations[x].Path);
#endif	

#if 0
	pause("Optimized Drivers : %d", Config.OptDriverCount );
	for(x = 0; x < Config.OptDriverCount; x++)
		pause("%s %s", Config.OptDrivers[x].Desc, Config.OptDrivers[x].Path);
#endif

  ReadMissing();  
} 

void parse_config(char *config)
{
	int len = strlen(config);
	int pos = 0;
	char ident[256];
	FILTER_GROUP *fg;
	char DefaultDriverLocation[81]="";

  while(1)
  {
		skip_white(config, &pos);
		get_token(config, &pos, ident);	
		
		if(pos >= len || ident[0] == 0)
		  break;

		if(strcasecmp(ident, "DEFAULT-FILTER-GROUP") == 0)
		  get_value(config, &pos, Config.DefaultFilterGroup);
		else if(strcasecmp(ident, "DEFAULT-FILTER") == 0)
		  get_value(config, &pos, Config.DefaultFilter);
		else if(strcasecmp(ident, "Repeat-Delay") == 0)
		  Config.RepeatDelay = GetIntValue(config, &pos);
		else if(strcasecmp(ident, "Repeat-Rate") == 0)
		  Config.RepeatRate = GetIntValue(config, &pos);
		else if(strcasecmp(ident, "Auto-FXE-Filter") == 0)
		  Config.AutoFXEFilter = GetBooleanValue(config, &pos);
		else if(strcasecmp(ident, "Auto-Year-Filter") == 0)
		  Config.AutoYearFilter = GetBooleanValue(config, &pos);
		else if(strcasecmp(ident, "Auto-CloneOf-Filter") == 0)
		  Config.AutoCloneOfFilter = GetBooleanValue(config, &pos);
    else if(strcasecmp(ident, "Auto-Genre-Filter") == 0)
      Config.AutoGenreFilter = GetBooleanValue(config, &pos);
    else if(strcasecmp(ident, "Auto-Category-Filter") == 0)
      Config.AutoCategoryFilter = GetBooleanValue(config, &pos);
    else if(strcasecmp(ident, "Auto-Manufacturer-Filter") == 0)
      Config.AutoManufacturerFilter = GetBooleanValue(config, &pos);
		else if(strcasecmp(ident, "Update-Image-Freq") == 0)
		  Config.UpdateImageFreq = GetIntValue(config, &pos);
		else if(strcasecmp(ident, "Filter-Missing-Games") == 0)
		  Config.FilterMissingGames = GetBooleanValue(config, &pos);
		else if(strcasecmp(ident, "Default-Driver-Location") == 0)
		  get_value(config, &pos, DefaultDriverLocation);
		else if(strcasecmp(ident, "Driver-Locations") == 0)
		  ReadDriverLocations(config, &pos);
		else if(strcasecmp(ident, "Game-Optimized-Drivers") == 0)
		  ReadGameOptimizedDrivers(config, &pos);
		else if(strcasecmp(ident, "Force-BLU-Plus")== 0)
		{
		  extern int GP32_NEW_BLU;
 		  GP32_NEW_BLU=GetBooleanValue(config, &pos);
		}
		else if(strcasecmp(ident, "Filter-Group") == 0)
		{
		  get_value(config, &pos, ident);
			fg = add_filter_group(ident);
			read_filters(fg, config, &pos);
		}
		else
		  pause("Unknown ident %s", ident);
		  
  }

#if 0
pause("Default Filter Group %s", Config.DefaultFilterGroup);
pause("Default Filter %s", Config.DefaultFilter);
pause("Repeat Delay %d", Config.RepeatDelay);
pause("Repeat Rate %d", Config.RepeatRate);
#endif

  if(DefaultDriverLocation[0] != 0)
  	SetDriverLocation(DefaultDriverLocation);
}

FILTER_GROUP * add_filter_group(char *FilterGroupDesc)
{
  if(Config.GroupCount >= Config.GroupsAllocated)
  {
  	FILTER_GROUP *temp;
  	int x;
  	
  	Config.GroupsAllocated += 10;
  	temp = (FILTER_GROUP *)malloc(sizeof(FILTER_GROUP) * Config.GroupsAllocated);
  	memset(temp, 0, sizeof(FILTER_GROUP) * Config.GroupsAllocated);
  	
  	for(x=0; x < Config.GroupCount; x++)
  		temp[x] = Config.Groups[x];
  		
    free(Config.Groups);		
    Config.Groups = temp;		
  }

  strcpy(Config.Groups[Config.GroupCount].FilterGroupDesc, FilterGroupDesc);
  ++Config.GroupCount;
  
  return(Config.Groups + Config.GroupCount-1);
}

void add_filter(FILTER_GROUP *fg, FILTER *filter)
{
	if(fg->FilterCount >= fg->FiltersAllocated)
	{
		FILTER *temp;
		int x;
		
		fg->FiltersAllocated += 10;
		temp = (FILTER *)malloc(sizeof(FILTER) * fg->FiltersAllocated);
		
		for(x=0; x < fg->FilterCount; x++)
			temp[x] = fg->Filters[x];
			
	  if(fg->Filters) free(fg->Filters);
	  fg->Filters = temp;		
	}
	
	strcpy(fg->Filters[fg->FilterCount].FilterDesc, filter->FilterDesc);
	fg->Filters[fg->FilterCount].Field = filter->Field;
	fg->Filters[fg->FilterCount].Op = filter->Op;
  fg->Filters[fg->FilterCount].Values = filter->Values;    /* move allocated pointer to new structure */
	
	++fg->FilterCount;
}

int GetIntValue(char *config, int *pos)
{
	char temp[256];
	get_value(config, pos, temp);
	return atoi(temp);
}

int GetBooleanValue(char *config, int *pos)
{
	char temp[256];
	get_value(config, pos, temp);
	
	if(strcasecmp(temp, "YES") == 0)
	  return 1;
	  
	if(strcasecmp(temp, "1") == 0)
	  return 1;
	
	if(strcasecmp(temp, "TRUE") == 0)
	  return 1;
	  
	if(strcasecmp(temp, "NO") == 0)
	  return 0;

	if(strcasecmp(temp, "FALSE") == 0)
	  return 0;
	  
	if(strcasecmp(temp, "0") == 0)
	  return 0;
	  
  pause("%s not a boolean value");	 
  return 0; 
}



char *get_value(char *config, int *pos, char *dest)
{
	char c;
	
	skip_white(config, pos);
	
	c = config[*pos];
	if(c == '=')
	{
		++(*pos);
		skip_white(config, pos);
	}
	
	get_token(config, pos, dest);
	return dest;
}

int get_operatorcode(char *config, int *pos)
{
	char temp1[25], temp2[25];
	
	get_token(config, pos, temp1);
	strupr(temp1);
	
	if(strcmp(temp1, "STARTS") == 0 || strcmp(temp1, "STARTING") == 0)
	{
		get_token(config, pos, temp2);
		strupr(temp2);
		
		if(strcmp(temp2, "WITH") != 0)
		  doerror("syntax error: STARTS and STARTING must be followed by WITH");

	  strcpy(temp1, "STARTS WITH");
	}
	  
	return(opstring_to_code(temp1));
}

int get_fieldcode(char *config, int *pos)
{
	char temp[25];
	
	get_token(config, pos, temp);
	strupr(temp);
	
	return(fieldstring_to_code(temp));
}

int fieldstring_to_code(char *str)
{
  if(strcasecmp(str, "ALL") == 0)
    return FIELD_ALL;

	if(strcasecmp(str, "DESC") == 0)
	  return FIELD_DESC;

	if(strcasecmp(str, "MANUFACTURER") == 0)
	  return FIELD_MANUFACTURER;

	if(strcasecmp(str, "FXE") == 0)
	  return FIELD_FXE;

	if(strcasecmp(str, "YEAR") == 0)
	  return FIELD_YEAR;

	if(strcasecmp(str, "CLONEOF") == 0)
	  return FIELD_CLONEOF;

	if(strcasecmp(str, "GAME") == 0)
	  return FIELD_GAME;

	if(strcasecmp(str, "FILE") == 0)   /* alias to GAME */
	  return FIELD_GAME;

	if(strcasecmp(str, "GENRE") == 0)
	  return FIELD_GENRE;

	if(strcasecmp(str, "CATEGORY") == 0)
	  return FIELD_CATEGORY;

  doerror("Unknown field |%s|", str);
  
  return -1;
}

void get_token(char *config, int *pos, char *ident)
{
	char c;
	int size = 0, quoted = 0, paren = 0;
	
	ident[0] = 0;
	
	skip_white(config, pos);
	
	if(config[*pos] == ',')
	{
		ident[0] = config[*pos];
		ident[1] = 0;
		++(*pos);
		return;
	}
	
	
	if(config[*pos] == '"')
	{
	  quoted = 1;
	  ++(*pos);
	}
	
	if(config[*pos] == '(')
	{
	  paren = 1;
	  ++(*pos);
	}
	
	

	while(1)
	{
		c = config[*pos];
		if(c == 0) break;

		if(paren)
		{
    	if(c == 13 || c == 10)
    	  doerror("Unterminated string");
    	  
    	if(c == ')')
    	{
    		++(*pos);
    		break;
    	}  
		  ident[size++] = c;
    }
    else if(quoted)
    {
    	if(c == 13 || c == 10)
    	  doerror("Unterminated string");
    	  
    	if(c == '"')
    	{
    		++(*pos);
    		break;
    	}  
		  ident[size++] = c;
    }
    else
    {		
			if(isalpha(c) || isdigit(c) || c == '-' || c == '_')
			  ident[size++] = c;
			else
			  break;
    }
		  
		++(*pos);
	}
	ident[size] = 0;
}


void skip_white(char *config, int *pos)
{
	char c;
	while(1)
	{
		c = config[*pos];
		if(c == 0) return;
		
		if(c == 32 || c == 8 || c == 9 || c == 12 || c == 13 || c == 10)
		  ++(*pos);
		else 
		{
			if(c == '#')
			  skip_line(config, pos);
			else
		    return;
		}
	}
}

void skip_line(char *config, int *pos)
{
	char c;
	while(1)
	{
		c = config[*pos];
		if(c == 0) return;
		
		if(c != 10 && c != 13)
		  ++(*pos);
		else
		  return;
	}
}



void read_filters(FILTER_GROUP *fg, char *config, int *pos)
{
	FILTER filter;
	char temp[1024];
	int SavePos;
	
	skip_white(config, pos);
	
	if(config[*pos] != '{') 
	{
		pause("Current : |%-20.20s|", config + *pos);
	  doerror("Error in frontend.cfg, excepted {");
	}

  while(1)
  {
	  ++(*pos);
	  skip_white(config, pos);
	  
	  if(config[*pos] == 0)
	    doerror("Unexpected end of file, expecting }");
	  
	  if(config[*pos] == '}')
	  {
	  	++(*pos);
	    return;
	  }

    get_token(config, pos, filter.FilterDesc);
    filter.Field = get_fieldcode(config, pos);
    filter.Op = get_operatorcode(config, pos);
    
    get_token(config, pos, temp);
    parse_filter_values(&filter, temp);
    
// REGEXPS can have an EXCLUDE option
    SavePos = *pos;
    get_token(config, pos, temp);
    if(strcasecmp(temp, "EXCLUDE") == 0)
    {
    	get_token(config, pos, temp);
    	
	  	filter.Values[1] = regcomp(strupr(temp));
    	if(!filter.Values[1])
  	    pause("Invalid EXCLUDE regexp %s buffer", temp);
    }
    else
      *pos = SavePos;
// end of exclude      
    
    
    add_filter(fg, &filter);

#if 0
		pause("Desc : %s\n", filter->desc);    
		pause("Field : %s\n", filter->field);    
		pause("Operator : %s\n", filter->op);    
		pause("Value : %s\n", filter->value);    
#endif
  }
}

void free_filters_groups(void)
{
	int x;
	
	if(Config.GroupsAllocated == 0)
	  Config.Groups = NULL;
  
  for(x=0; x < Config.GroupCount; x++)
  {
  	if(Config.Groups[x].Filters)
  	  free(Config.Groups[x].Filters);
  	Config.Groups[x].Filters = NULL;
  	
  	Config.Groups[x].FiltersAllocated = 0;
  	Config.Groups[x].FilterCount = 0;
  }
  
  if(Config.Groups) free(Config.Groups);
  Config.Groups = NULL;
  Config.GroupsAllocated = 0;
  Config.GroupCount = 0;
}


void dump_filter_groups(void)
{
	int x, y, z;
	FILTER *f;
  printf("There are %d filters  (allocated = %d)\n\n", Config.GroupCount, Config.GroupsAllocated);
  
  for(x=0; x < Config.GroupCount; x++)
  {
  	printf("Group %d : %s\n", x+1, Config.Groups[x].FilterGroupDesc);
  	
  	for(y=0; y < Config.Groups[x].FilterCount; y++)
  	{
  		f = Config.Groups[x].Filters + y;
  		printf("  Desc:%s Field:%s Op:%d  values: ( ", f->FilterDesc, f->Field, f->Op);
  		
  		for(z = 0; f->Values[z]; z++)
  		  printf("%s ", f->Values[z]);
  		  
  		printf(")\n");
  		
  		
  	}
  	printf("\n\n");
  }
}

int opstring_to_code(char *opcode)
{
	if(strcasecmp(opcode, "STARTS WITH") == 0)
	  return OP_STARTSWITH;
	  
	if(strcasecmp(opcode, "CONTAINS") == 0)
	  return OP_CONTAINS;
	  
	if(strcasecmp(opcode, "EQUALS") == 0)
	  return OP_EQUALS;
	  
	if(strcasecmp(opcode, "REGEXP") == 0)
	  return OP_REGEXP;
	  
	if(strcasecmp(opcode, "STARTING WITH") == 0)
	  return OP_STARTSWITH;

  doerror("Unknown opcode %s", opcode);
  
  return(-1);
}


void parse_filter_values(FILTER *filter, char *buffer)
{
	int x=0, commas;
	int pos = 0, count = 0;
	char token[256];
	
	filter->Values = NULL;

  if(filter->Op == OP_REGEXP)
  {
  	filter->Values = malloc( sizeof(void *) * 3);
  	filter->Values[0] = NULL;
  	filter->Values[1] = NULL;
  	filter->Values[2] = NULL;
  	
  	filter->Values[0] = regcomp(strupr(buffer));
  	if(!filter->Values[0])
  	  pause("Invalid regexp %s buffer", buffer);
  	  
/*pause("compile: %x %s", filter->rvalues  	 */
  	return;
  }

  commas = 0;
	while(buffer[x] != 0)
	{
		if(buffer[x] == ',')
		  commas++;
		x++;
	}
	++commas;
	
	filter->Values = malloc( sizeof(char **) * commas + 1);
	for(x = 0; x <= commas; x++)
	  filter->Values[x] = NULL;

	pos = 0;
	while(1)
	{
	  get_token(buffer, &pos, token);
	  if(strcmp(token, ",") == 0) continue;
	  
	  if(token[0] == 0) break;
	  strupr(token);  /* case insensitive searches */
	  
	  filter->Values[count++] = strdup(token);
	}
}

void InitalizeConfig(void)
{
	if(initialized)
	{
		if(Config.Groups) free(Config.Groups);
		if(Config.DriverLocations ) free(Config.DriverLocations);
		if(Config.OptDrivers) free(Config.OptDrivers);
	}

  Config.DefaultFilterGroup[0]=0;
  Config.DefaultFilter[0] = 0;
  
  Config.RepeatDelay = 500;
  Config.RepeatRate = 50;
  Config.UpdateImageFreq = 250;
  
  Config.AutoFXEFilter = 0;
  Config.AutoYearFilter = 1;
  Config.AutoCloneOfFilter = 0;
  Config.AutoGenreFilter = 1;
  Config.AutoCategoryFilter = 0;
  Config.AutoManufacturerFilter = 1;

	Config.Groups = NULL;
	Config.GroupCount = 0;
	Config.GroupsAllocated = 0;
	Config.CurrentGroup = -1;

	Config.DriverLocations = NULL;
	Config.DriverLocationsCount = 0;
	Config.DriverLocationsAllocated = 0;	  
	Config.CurrentDriverLocation = -1;
	
	Config.OptDrivers = NULL;
	Config.OptDriverCount = 0;
	Config.OptDriverAllocated = 0;
	Config.CurrentOptDriver = -1;
	Config.OptDriverDisabled = 0; 
	
	Config.FilterMissingGames = 1;
}

void ReadDriverLocations(char *config, int *pos)
{
  DRIVERLOC DriverLoc;
	
	skip_white(config, pos);
	
	if(config[*pos] != '{') 
	{
		pause("Current : |%-20.20s|", config + *pos);
	  doerror("Error reading driver locations, excepted {");
	}

  while(1)
  {
	  ++(*pos);
	  skip_white(config, pos);
	  
	  if(config[*pos] == 0)
	    doerror("Unexpected end of file, expecting }");
	  
	  if(config[*pos] == '}')
	  {
	  	++(*pos);
	    return;
	  }

		get_token(config, pos, DriverLoc.Desc);
		get_token(config, pos, DriverLoc.Path);
		
		if(DriverLoc.Path[strlen(DriverLoc.Path)] != '/')
		  strcat(DriverLoc.Path, "/");
    
    AddDriverLocation(&DriverLoc);
  }
}

void AddDriverLocation(DRIVERLOC *DriverLoc)
{
	if(Config.DriverLocationsCount >= Config.DriverLocationsAllocated)
	{
		DRIVERLOC *temp;
		int x;
		
		Config.DriverLocationsAllocated += 10;
		temp = (DRIVERLOC *)malloc(sizeof(DRIVERLOC) * Config.DriverLocationsAllocated);
		
		for(x = 0; x < Config.DriverLocationsCount; x++)
		  temp[x] = Config.DriverLocations[x];
		 
		if(Config.DriverLocations) free(Config.DriverLocations);
		Config.DriverLocations = temp;  
	}
	
	Config.DriverLocations[Config.DriverLocationsCount] = *DriverLoc;
	++Config.DriverLocationsCount;
}

void ReadGameOptimizedDrivers(char *config, int *pos)
{
  DRIVERLOC DriverLoc;
	
	skip_white(config, pos);
	
	if(config[*pos] != '{') 
	{
		pause("Current : |%-20.20s|", config + *pos);
	  doerror("Error reading game optimized drivers, excepted {");
	}

  while(1)
  {
	  ++(*pos);
	  skip_white(config, pos);
	  
	  if(config[*pos] == 0)
	    doerror("Unexpected end of file, expecting }");
	  
	  if(config[*pos] == '}')
	  {
	  	++(*pos);
	    return;
	  }

		get_token(config, pos, DriverLoc.Desc);
		get_token(config, pos, DriverLoc.Path);
    
    AddGameOptimizedDriver(&DriverLoc);
  }
}

void AddGameOptimizedDriver(DRIVERLOC *DriverLoc)
{
	if(Config.OptDriverCount >= Config.OptDriverAllocated)
	{
		DRIVERLOC *temp;
		int x;
		
		Config.OptDriverAllocated += 10;
		temp = (DRIVERLOC *)malloc(sizeof(DRIVERLOC) * Config.OptDriverAllocated);
		
		for(x = 0; x < Config.OptDriverCount; x++)
		  temp[x] = Config.OptDrivers[x];
		 
		if(Config.OptDrivers) free(Config.OptDrivers);
		Config.OptDrivers = temp;  
	}
	
	Config.OptDrivers[Config.OptDriverCount] = *DriverLoc;
	++Config.OptDriverCount;
}

void SetDriverLocation(char *LocDesc)
{
	int x;
	for(x = 0; x < Config.DriverLocationsAllocated; x++)
	{
		if(strcasecmp(Config.DriverLocations[x].Desc, LocDesc) == 0)
		{
			Config.CurrentDriverLocation = x;
			return;
		}
	}
	
	pause("Unable to set driver location %s", LocDesc);
}

