#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

#include "config.h"
#include "util.h"
#include "filter.h"
#include "video.h"
#include "drivers.h"
#include "regexp.h"
#include "missinggames.h"
#include "quickfilter.h"



static int SearchHayStack(FILTER *filter, char *haystack);
static void GetTextToSearch(DRIVER *d, int field, char *buffer);


void SetFilterGroup(char *FilterGroupDesc)
{
	int x;
  for(x=0; x < Config.GroupCount; x++)
  {
  	if(strcasecmp(FilterGroupDesc, Config.Groups[x].FilterGroupDesc) == 0)
  	{
  		Config.CurrentGroup = x; 
    	ApplyFiltersToGameList();
  		return;
  	}
  }
  pause("Unable to set group |%s|", FilterGroupDesc);
}


void GoToNextFilterGroup(void)
{
	++Config.CurrentGroup;
	if(Config.CurrentGroup >= Config.GroupCount)
	  Config.CurrentGroup = 0;
	
	ApplyFiltersToGameList();
}

void GoToPrevFilterGroup(void)
{
	--Config.CurrentGroup;
	if(Config.CurrentGroup < 0)
	  Config.CurrentGroup = Config.GroupCount - 1;
	
	ApplyFiltersToGameList();
}



static void GetTextToSearch(DRIVER *d, int field, char *buffer)
{
	buffer[0] = 0;

  if(field == FIELD_DESC)	
	  strcpy(buffer, d->description);
	else if(field == FIELD_GAME)
	  strcpy(buffer, d->name);
	else if(field == FIELD_FXE)
	  strcpy(buffer, d->fxe);
	else if(field == FIELD_MANUFACTURER)
    strcpy(buffer, d->manufacturer);	  
  else if(field == FIELD_YEAR)
    strcpy(buffer, d->year);
  else if(field == FIELD_CLONEOF)
    strcpy(buffer, d->cloneof);
  else if(field == FIELD_GENRE)
    strcpy(buffer, d->genre);
  else if(field == FIELD_CATEGORY)
    strcpy(buffer, d->category);
	else if(field == FIELD_ALL)
	  sprintf(buffer, "%s %s %s", d->description, d->fxe, d->name);
  else
	  doerror("Unknown field '%s'", field);
	
	strupr(buffer);
}

int CheckFilter(DRIVER *d)
{
	FILTER *Filter = CurrentFilter();
	char haystack[1024];
	
	if(Config.FilterMissingGames)
	{
		if(IsGameMissing(d->name))
		  return 0;
	}
	
	if(!Filter) return 1;  /* no filter, then match all */
	
	GetTextToSearch(d, Filter->Field, haystack);
  return(SearchHayStack(Filter, haystack));
}


static int SearchHayStack(FILTER *filter, char *haystack)
{
	int x;
	
	if(filter->Op == OP_REGEXP)
	{
		if(filter->Values[0] == NULL) doerror("RegExp value is NULL");
		
		if(regexec(filter->Values[0], haystack))
		{
			/* check excludes before returning a find*/
			if(filter->Values[1])
			{
    		if(regexec(filter->Values[1], haystack))
    		  return(0);  /* this find has been excluded */
			}
		  return(1);
		}
	  return(0);
	}
	
	
	for(x=0; filter->Values[x]; x++)
	{
		if(filter->Op == OP_STARTSWITH)
		{
			if(strncasecmp(filter->Values[x], haystack, strlen(filter->Values[x])) == 0)
			  return(1);
		}
		
		if(filter->Op == OP_CONTAINS)
		{
			if(strstr(haystack, filter->Values[x]) != NULL)
			  return(1);
		}
		
		if(filter->Op == OP_EQUALS)
		{
			if(strcasecmp(filter->Values[x], haystack) == 0)
			  return(1);
		}
	}
	return(0);  /* no match */
}

FILTER_GROUP * CurrentFilterGroup(void)
{
	if(Config.GroupCount == 0)
	  return NULL;
	  
	if(Config.CurrentGroup >= Config.GroupCount || Config.CurrentGroup < 0) 
	  Config.CurrentGroup = 0;

  /* no filter groups? */
	if(Config.CurrentGroup >= Config.GroupCount) return NULL;  

	return(Config.Groups + Config.CurrentGroup);
}

FILTER * CurrentFilter(void)
{
	FILTER_GROUP *FG = CurrentFilterGroup();
	if(!FG) return(NULL);
	
  if(QuickFilterStr)
  {
  	if(QuickFilterStr[0]) 
  	  return(QuickFilterObj);
  }
	
	
	if(FG->FilterCount < 1) return(NULL);
	return(FG->Filters + FG->CurrentFilter);
}


void SetFilter(char *FilterDesc)
{
	int x;
	FILTER_GROUP *FG = CurrentFilterGroup();
	if(!FG)
	{
		pause("Unable to set filter to %s", FilterDesc);
		return;
	}
	
	for(x = 0; x < FG->FilterCount; x++)
	{
		if(strcasecmp(FG->Filters[x].FilterDesc, FilterDesc) == 0)
		{
			FG->CurrentFilter = x;
			return;
		}
	}
	
	pause("Unable to find filter %s", FilterDesc);
}

void GoToPrevFilter(void)
{
	FILTER_GROUP *FG = CurrentFilterGroup();
	if(!FG) return;
	
  --FG->CurrentFilter;
	if(FG->CurrentFilter >= FG->FilterCount || FG->CurrentFilter < 0)
		FG->CurrentFilter = FG->FilterCount-1;
	  
	ApplyFiltersToGameList();
}

void GoToNextFilter(void)
{
	FILTER_GROUP *FG = CurrentFilterGroup();
	if(!FG) return;
	
  ++FG->CurrentFilter;

	if(FG->CurrentFilter >= FG->FilterCount || FG->CurrentFilter < 0)
		FG->CurrentFilter = 0;

	ApplyFiltersToGameList();
}


void ApplyFiltersToGameList(void)
{
  int i;
  FILTER * Filter = CurrentFilter();

  if(!Filter) return;

/*  Filter->SelectedGameFilterID =  */
  Filter->GamesMatchingFilter = 0;
  Filter->__SelectedGameID = -1;
  Filter->__TotalGames = NUMGAMES;
  
  for (i=0;i<NUMGAMES;i++)
  {
    drivers[i].available=0;

    if(CheckFilter(drivers + i))
    {
      drivers[i].available=1;
      ++Filter->GamesMatchingFilter;
    }
  }
/*  
  if(Filter->SelectedGameFilterID < 0 || Filter->SelectedGameFilterID >= Filter->GamesMatchingFilter)
*/  
}

void SelectNextGame(void)
{
	FILTER *Filter = CurrentFilter();
	if(!Filter) return;
	
	Filter->SelectedGameFilterID++;
	Filter->__SelectedGameID = -1;  /* this gets updated with the correct value when I draw the screen */
	
  if(Filter->SelectedGameFilterID >= Filter->GamesMatchingFilter)
    Filter->SelectedGameFilterID = Filter->GamesMatchingFilter - 1;
    
  if(Filter->SelectedGameFilterID < 0)
    Filter->SelectedGameFilterID = 0;  

	if(Filter->GamesMatchingFilter <= 0)
		Filter->SelectedGameFilterID = -1;
}

void SelectPrevGame(void)
{
	FILTER *Filter = CurrentFilter();
	if(!Filter) return;
	
	Filter->SelectedGameFilterID--;
	Filter->__SelectedGameID = -1;  /* this gets updated with the correct value when I draw the screen */
	
  if(Filter->SelectedGameFilterID >= Filter->GamesMatchingFilter)
    Filter->SelectedGameFilterID = Filter->GamesMatchingFilter - 1;
    
  if(Filter->SelectedGameFilterID < 0)
    Filter->SelectedGameFilterID = 0;  

	if(Filter->GamesMatchingFilter <= 0)
		Filter->SelectedGameFilterID = -1;
}

char * GetFilterState(char *state)
{
	int x, y;
	char temp[25];
	FILTER_GROUP *Group;
	
	sprintf(state, "%d", Config.CurrentGroup);
	
	for(x = 0; x < Config.GroupCount; x++)
	{
		Group = Config.Groups + x;
		sprintf(temp, ",G,%d", x, Group->CurrentFilter);
		strcat(state, temp);
		
		
		for(y = 0; y < Group->FilterCount ; y++)
		{
			sprintf(temp, ",%d", Group->Filters[y].SelectedGameFilterID);
			strcat(state, temp);
		}
	}
	return(state);
}

void SetFilterState(char *state)
{
	char *tok;
	int GroupPos, FilterPos;
	FILTER_GROUP *Group=NULL;
	
	tok = strtok(state, ",");
	if(!tok) return;
	
	Config.CurrentGroup = atoi(tok);
	if(Config.CurrentGroup >= Config.GroupCount)
	{
	  pause("Error restoring filter state (1)");
	  return;
	}
	
	GroupPos = -1;
	FilterPos = -1;
	while((tok = strtok(NULL, ",")) != NULL)
	{
		if(tok[0] == 'G')
		{
			++GroupPos;
			FilterPos = 0;
    	Group = Config.Groups + GroupPos;
    	continue;
		}
		if(GroupPos >= Config.GroupCount || GroupPos < 0)
		{
		  pause("Error restoring filter state (2)");
		  return;
		}
		if(FilterPos >= Group->FilterCount)
		{
		  pause("Error restoring filter state (3)");
		  return;
		}
		Group->Filters[FilterPos].SelectedGameFilterID = atoi(tok);
	}
}

/* to be called when the program is first loaded  */
/* either restore state from params passed to the */
/* program, or use default values                 */
void SetInitialFilterState(void)
{
	char *env, fn[256], rom[256], state[1024];
	FILTER_GROUP *FG;
	int x, y;

  fn[0] = 0;
  rom[0] = 0;
	state[0] = 0;
	
	env = AppPath();
	sscanf(env, "%s %s %s", fn, rom, state);

#if 0
	pause("env = %s", env);
	pause("fn = %s", fn);
	pause("rom = %s", rom);
	pause("state = |%s|", state);
#endif	

/* Initialize current positions to 0 */
  Config.CurrentGroup = 0;
  for(x = 0; x < Config.GroupCount; x++)
  {
  	FG = Config.Groups + x;
  	FG->CurrentFilter = 0;
  	
  	for(y = 0; y < FG->FilterCount; y++)
  	  FG->Filters[y].SelectedGameFilterID = 0;
  }
	
	if(state[0] != 0)
		SetFilterState(state);
  else
  {
  	if(Config.DefaultFilterGroup[0] != 0)
  	{
  	  SetFilterGroup(Config.DefaultFilterGroup);
  	  
  	  if(Config.DefaultFilter[0] != 0)
  	    SetFilter(Config.DefaultFilter);
  	}
  }
  
	ApplyFiltersToGameList();
}


