#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <math.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

//#include <SDL_image.h>

#include "video.h"
#include "util.h"
#include "filter.h"
#include "selector.h"
#include "StandardHandleEvent.h"
#include "missinggames.h"
#include "quickfilter.h"

void DrawSelectorMenu(void);
void ValidatePositions(void);
void DrawMenuItem(int idx, int *XPos);

static int onKeyDown(SDLKey key, Uint8 *KeyState);
static int onIdle(void);


#undef MAX
#define MAX(a,b)	((a) > (b) ? (a) : (b))

#define ADV_MENU_ITEMS 5

	
	
char *menu_items[] = { "Groups", "Filters", "Adv", 0 };
int ItemPos, MenuPos;

static SDL_Surface *BG;
static SDL_Rect BG_R = { 0, 0, 320, 240 };

void SelectorMenu(void)
{
	/* copy the existing screen surface, it will be used to 
	 * restore the background when the menu moves around */
	SDL_Surface *tmp = SDL_CreateRGBSurface(0, 320, 240, 8, 0, 0, 0, 0);
	if(!tmp) pause("BG is null");
	else
	{
		BG = SDL_DisplayFormat(tmp);
		SDL_SetColorKey(BG ,SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(screen->format, 0x80, 0x80, 0x80));
		SDL_FreeSurface(tmp);
		SDL_BlitSurface(screen, NULL, BG, &BG_R);
	}
	
	MenuPos = 0;
  ItemPos = Config.CurrentGroup;
  
  DrawSelectorMenu();
  StandardHandleEvent(onKeyDown, onIdle);
  
  SDL_FreeSurface(BG);
  BG = NULL;
  
  ClearEvents();
}



void ValidatePositions(void)
{
	int CountOfMenuItems;
	
	for(CountOfMenuItems =0; menu_items[CountOfMenuItems]; CountOfMenuItems++)
	  ;
	  
	if(MenuPos >= CountOfMenuItems)
	  MenuPos = 0;
	  
	if(MenuPos < 0)
	  MenuPos = CountOfMenuItems - 1;
	
  if(MenuPos == 0)
  {
  	if(ItemPos < 0)
  	  ItemPos = Config.GroupCount - 1;
  	  
  	if(ItemPos >= Config.GroupCount)
  	  ItemPos = 0;  
  }	
}


void DrawSelectorMenu(void)
{
	int idx, XPos;
	SDL_Rect r = { 0, 0, 320, 20 };
	
  SDL_BlitSurface(BG, NULL, screen, &BG_R); 
	
	SDL_FillRect(screen, &r, 55);

  	
  XPos = 10;
	for(idx = 0; menu_items[idx]; idx++)
	  DrawMenuItem(idx, &XPos);

  SDL_Flip(screen);
}

void DrawMenuItem(int idx, int *XPos)
{
	SDL_Rect R1, R2;
	Uint32 c;
	int item_rows, item_width=0, x;
	int Width = (strlen(menu_items[idx]) * 9);
	
	if(idx == MenuPos)
	{
		if(idx == 0)  /* Group */
		{
			int font_height = 13;
			int font_width = 7;
			
      gfxPrimitivesSetFont(&SDL_gfx_font_7x13B_fnt, font_width, font_height);
  		
		  item_rows = Config.GroupCount;
			for(x=0; x < Config.GroupCount; x++)
			  item_width = MAX(item_width, strlen(Config.Groups[x].FilterGroupDesc));
  		
	    c = SDL_MapRGBA(screen->format, 197, 192, 188, 255); 

	    R1 = RECT(*XPos, 21, (item_width * font_width) + 10, (item_rows * font_height) + 10);
			SDL_FillRect(screen, &R1, c);
	  	rectangleRGBA(screen, R1.x, R1.y, R1.x+R1.w, R1.y+R1.h, 0, 0, 0, 255);  

			for(x=0; x < Config.GroupCount; x++)
			{
    		SetFontColor(0, 0, 0, 255);
				if(x == ItemPos)
				{
					SetFontColor(255, 255, 255, 255);
					R2 = RECT(R1.x, R1.y + 5 + (x * font_height), R1.w, font_height);
       		SDL_FillRect(screen, &R2, 70);
				}
				TextAt(R1.x + 5, R1.y + 5 + (x * font_height), Config.Groups[x].FilterGroupDesc);  
			}
		}
		
		if(idx == 1)  /* filters */
		{
			int font_height = 13;
			int font_width = 7;
			FILTER_GROUP *FG = CurrentFilterGroup();
			
      gfxPrimitivesSetFont(&SDL_gfx_font_7x13B_fnt, font_width, font_height);
  		
		  item_rows = FG->FilterCount;
			for(x=0; x < FG->FilterCount; x++)
  	    item_width = MAX(item_width, strlen(FG->Filters[x].FilterDesc ));
  		
	    c = SDL_MapRGBA(screen->format, 197, 192, 188, 255); 

	    R1 = RECT(*XPos, 21, (item_width * font_width) + 10, (item_rows * font_height) + 10);
			SDL_FillRect(screen, &R1, c);
	  	rectangleRGBA(screen, R1.x, R1.y, R1.x+R1.w, R1.y+R1.h, 0, 0, 0, 255);  

			for(x=0; x < FG->FilterCount; x++)
			{
    		SetFontColor(0, 0, 0, 255);
				if(x == ItemPos)
				{
					SetFontColor(255, 255, 255, 255);
					R2 = RECT(R1.x, R1.y + 5 + (x * font_height), R1.w, font_height);
       		SDL_FillRect(screen, &R2, 70);
				}
				TextAt(R1.x + 5, R1.y + 5 + (x * font_height), FG->Filters[x].FilterDesc);  
			}
		}
		
			   
		if(idx == 2)  /* Adv */
		{
			int font_height = 13;
			int font_width = 7;
			char items[10][100];
			
      gfxPrimitivesSetFont(&SDL_gfx_font_7x13B_fnt, font_width, font_height);

		  item_rows = ADV_MENU_ITEMS;
  		sprintf(items[0], "Quick Filter");
  		sprintf(items[1], "Driver: %s", Config.DriverLocations[Config.CurrentDriverLocation].Desc);
  		sprintf(items[2], "Optimal Driver: %s", Config.OptDriverDisabled ? "Disabled" : "Enabled");
  		sprintf(items[3], "Filter Missing Games : %s", Config.FilterMissingGames ? "Yes" : "No");
  		sprintf(items[4], "Rebuild MISSING.CFG");
  		
		  for(x = 0; x < item_rows; x++)
	      item_width = MAX(item_width, strlen(items[x]));

	    c = SDL_MapRGBA(screen->format, 197, 192, 188, 255); 

	    R1 = RECT(*XPos, 21, (item_width * font_width) + 10, (item_rows * font_height) + 10);
	    
	    if(R1.x + R1.w > 320) R1.x = 320 - R1.w - 5;
	    
			SDL_FillRect(screen, &R1, c);
	  	rectangleRGBA(screen, R1.x, R1.y, R1.x+R1.w, R1.y+R1.h, 0, 0, 0, 255);  

			for(x=0; x < item_rows; x++)
			{
    		SetFontColor(0, 0, 0, 255);
				if(x == ItemPos)
				{
					SetFontColor(255, 255, 255, 255);
					R2 = RECT(R1.x, R1.y + 5 + (x * font_height), R1.w, font_height);
       		SDL_FillRect(screen, &R2, 70);
				}
				TextAt(R1.x + 5, R1.y + 5 + (x * font_height), items[x]);  
			}
		}
		
		/* draw the outline of the menu item */
		R1 = RECT(*XPos, 0, Width, 20);
		SDL_FillRect(screen, &R1, 70);
	}

	
  gfxPrimitivesSetFont(&SDL_gfx_font_9x18B_fnt,9,18);
  SetFontColor(255, 255, 0, 255);
	TextAt(*XPos, 0, menu_items[idx]);
	*XPos += (Width + 18);
}


static int onKeyDown(SDLKey key, Uint8 *KeyState)
{
  switch(key)
  {
  	case BUTTON_DOWN:	   
  	  if(MenuPos == 0)
  	  {
  	    GoToNextFilterGroup();
        ItemPos = Config.CurrentGroup;
  	  }
  	  if(MenuPos == 1)
  	  {
  	  	GoToNextFilter();
  	  	ItemPos = CurrentFilterGroup()->CurrentFilter;
  	  }  	  
  	  if(MenuPos == 2)
  	  {
  	  	ItemPos++;
  	  	if(ItemPos > ADV_MENU_ITEMS) ItemPos = 0;
  	  }
    	break;
    	
    case BUTTON_UP:
  	  if(MenuPos == 0)
  	  {
  	    GoToPrevFilterGroup();
        ItemPos = Config.CurrentGroup;
  	  }
  	  if(MenuPos == 1)
  	  {
  	  	GoToPrevFilter();
  	  	ItemPos = CurrentFilterGroup()->CurrentFilter;
  	  }
  	  if(MenuPos == 2)
  	  {
  	  	ItemPos--;
  	  	if(ItemPos < 0) ItemPos = ADV_MENU_ITEMS;
  	  }
    	break;
    	
    case BUTTON_LEFT:
      --MenuPos;
      ItemPos = 0;

      if(MenuPos == 0)  /* group */
        ItemPos = Config.CurrentGroup;
      if(MenuPos == 1)  /* filter */
        ItemPos = CurrentFilterGroup()->CurrentFilter;
      break;
      
    case BUTTON_RIGHT:
      ++MenuPos;
      ItemPos = 0;

      if(MenuPos == 0)  /* group */
        ItemPos = Config.CurrentGroup;
      if(MenuPos == 1)  /* filter */
        ItemPos = CurrentFilterGroup()->CurrentFilter;
      break;
      
    case 8:  /* left shoulder button */
      break;
      
    case 9: /* right shoulder button */
      break;
      
    case BUTTON_A:  /* A Fire Button*/
      if(MenuPos == 2) /* Advanced */
      {
      	if(ItemPos == 0)  /* Quick Filter */
      	{
      		/* erase menu, draw main screen */
				  SDL_BlitSurface(BG, NULL, screen, &BG_R); 
				  SDL_Flip(screen);
      		
      		QuickFilter();
      		ApplyFiltersToGameList();
      		return 1;
      		break;
      	}
      	if(ItemPos == 1)  /* Current Driver location */
      	{
      		Config.CurrentDriverLocation++;
      		if(Config.CurrentDriverLocation >= Config.DriverLocationsCount)
      		  Config.CurrentDriverLocation = 0;
      		break;  
      	}
      	if(ItemPos == 2)  /* Optimal drivers enabled | disabled */
      	{
      		Config.OptDriverDisabled = !Config.OptDriverDisabled;
      		break;
      	}
      	if(ItemPos == 3) /* Filter Missing Games Yes | No */
      	{
      		Config.FilterMissingGames = !Config.FilterMissingGames;
     			ApplyFiltersToGameList();
      		break;
      	}
      	
      	if(ItemPos == 4) /* rebuild MISSING.CFG */
      	{
      		BuildMissingCfg();
      		break;
      	}
      	
      }
      return 1;
      break;
      
    case BUTTON_B:   /* B Fire Button*/
      return 1;
      break;
      
    case BUTTON_START:  /* Start */
      break;
      
    case BUTTON_SELECT:  /* Select */
      break;
  }
  
 	ValidatePositions();
 	DrawSelectorMenu();
 	
 	return 0;
}

static int onIdle(void)
{
	return 0;
}



