#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

//#include <SDL_image.h>
#include <SDL_gfxPrimitives.h>

#include "video.h"
#include "util.h"
#include "filter.h"
#include "selector.h"
#include "menu.h"
#include "StandardHandleEvent.h"
#include "quickfilter.h"

static int onKeyDown(SDLKey key, Uint8 *KeyState);
static int onIdle(void);
static void DrawQuickFilter(void);

static SDL_Surface *BG;
static SDL_Rect BG_R = { 0, 0, 320, 240 };
static int CurPos = 0;

FILTER *QuickFilterObj = NULL;
static int QuickFilterPos;
char *QuickFilterStr="";  /* alias to QuickFilterObj->Values[0] */


#define CHARS_PER_LINE 6
#define SELECTOR_ITEMS 39
#define MAX_CHARS 25

void QuickFilter(void)
{
	/* copy the existing screen surface, it will be used to 
	 * restore the background when the menu moves around */
	SDL_Surface *tmp = SDL_CreateRGBSurface(0, 320, 240, 8, 0, 0, 0, 0);
	if(!tmp) pause("BG is null");
	else
	{
		BG = SDL_DisplayFormat(tmp);
		SDL_SetColorKey(BG ,SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(screen->format, 0x80, 0x80, 0x80));
		SDL_FreeSurface(tmp);
		SDL_BlitSurface(screen, NULL, BG, &BG_R);
	}
	
	if(!QuickFilterObj)
	{
  	QuickFilterObj = malloc(sizeof(FILTER));
  	QuickFilterObj->Field = FIELD_DESC;
  	QuickFilterObj->Op = OP_CONTAINS;
  	QuickFilterObj->Values = malloc( sizeof(char **) * 2 );
  	QuickFilterObj->Values[0] = malloc(MAX_CHARS);
  	QuickFilterObj->Values[1] = NULL;
  	
  	QuickFilterPos = 0;
  	QuickFilterStr = QuickFilterObj->Values[0];
  	QuickFilterStr[0] = 0;
	}

	CurPos = 0;	
  DrawQuickFilter();
  StandardHandleEvent(onKeyDown, onIdle);
  
  SDL_FreeSurface(BG);
  BG = NULL;
  
  ClearEvents();
}

void DrawQuickFilter(void)
{
	int x, xpos, ypos;
	char c, str[MAX_CHARS + 1];
	SDL_Rect WindowR = { 30, 40, 260, 130 };
	
  SDL_BlitSurface(BG, NULL, screen, &BG_R); 
	SDL_FillRect(screen, &WindowR, 55);
	
	SetFontColor(255, 255, 255, 255);
  gfxPrimitivesSetFont(&SDL_gfx_font_7x13B_fnt, 7, 13);
  
  for(x = 0; x < SELECTOR_ITEMS; x++)
  {
  	if(x < 36)
  	{
	  	xpos = (((x % CHARS_PER_LINE) + 1) * 14) + WindowR.x;
	  	ypos = (((x / CHARS_PER_LINE) + 1) * 15) + WindowR.y;
	  	
	  	c = 'A' + x;
	  	if(x >= 26) c = '0' + (x - 26);
	  	
	  	if(x == CurPos)
	    	rectangleColor(screen, xpos-3, ypos-1, xpos+7, ypos+13, 255);
	  	
		  TextAt(xpos, ypos, "%c", c);
  	}
  	else
  	{
      xpos = (((36 % CHARS_PER_LINE) + 1) * 14) + WindowR.x;

      if(x == 36) { xpos += 0;  strcpy(str, "bk"); }
      if(x == 37) { xpos += 20; strcpy(str, "clr"); }
      if(x == 38) { xpos += 50; strcpy(str, "ok"); }
      
	  	ypos = (((x / CHARS_PER_LINE) + 1) * 15) + WindowR.y;
	  	
	  	
	  	if(x == CurPos)
	    	rectangleColor(screen, xpos-3, ypos-1, xpos + (strlen(str) * 7), ypos+13, 255);
	  	
		  TextAt(xpos, ypos, "%s", str);
  	}
  }
  
  {
  	SDL_Rect R = RECT (WindowR.x + 100, WindowR.y + 15, 150, 40);
  	SetFontColor(0, 119, 51, 255);
    rectangleRGBA(screen, R.x, R.y, R.x + R.w, R.y + R.h, 68, 102, 136, 255);
    TextAt(R.x + 5, R.y + 5, "Quick Filter:");
    hlineRGBA(screen, R.x + 5, R.x + R.w - 10, R.y + 35, 68, 102, 136, 255);
    TextAt(R.x + 5, R.y + 22, QuickFilterObj->Values[0]);
  }
  
  	
  SDL_Flip(screen);
}

static int onKeyDown(SDLKey key, Uint8 *KeyState)
{
	char c[25];
	
  switch(key)
  {
  	case BUTTON_DOWN:	   
  	  CurPos += CHARS_PER_LINE;
    	break;
    	
    case BUTTON_UP:
      CurPos -= CHARS_PER_LINE;
    	break;
    	
    case BUTTON_LEFT:
      --CurPos;
      break;
      
    case BUTTON_RIGHT:
      ++CurPos;
      break;
      
    case BUTTON_LS:  /* left shoulder button */
      break;
      
    case BUTTON_RS: /* right shoulder button */
      break;
      
    case BUTTON_A:  /* A Fire Button*/
      if(CurPos == 38) 
      {
      	sprintf(QuickFilterObj->FilterDesc, "*%s*", QuickFilterStr);
      	return 1;   /* ok */
      }

      if(CurPos == 36)  /* back space */
      {
      	--QuickFilterPos; 
      	if(QuickFilterPos <= 0) QuickFilterPos = 0;
      	QuickFilterStr[QuickFilterPos] = 0;
      	break;
      }
      if(CurPos == 37)  /* clear */
      {
      	QuickFilterPos = 0;
      	QuickFilterStr[0] = 0;
      	break;
      }
      if(QuickFilterPos < MAX_CHARS)
      {
      	if(CurPos < 26)
      	  sprintf(c, "%c", 'A' + CurPos);
      	else
      	  sprintf(c, "%c", '0' + (CurPos - 26));
      	strcat(QuickFilterStr, c);
      	break;
      }
      break;
      
    case BUTTON_B:   /* B Fire Button*/
      break;
      
    case BUTTON_START:  /* Start */
      break;
      
    case BUTTON_SELECT:  /* Select */
      break;
  }
  
  QuickFilterPos = strlen(QuickFilterStr);

#if 0  
  if(CurPos < 0) CurPos = SELECTOR_ITEMS-1;
  if(CurPos >= SELECTOR_ITEMS) CurPos = 0;
#endif
  if(CurPos < 0) CurPos = 0;
  if(CurPos >= SELECTOR_ITEMS) CurPos = SELECTOR_ITEMS -1;

	DrawQuickFilter();
 	return 0;
}

static int onIdle(void)
{
	return 0;
}


