#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

#include "SDL.h"

#include "config.h"
#include "filter.h"
#include "video.h"
#include "util.h"


void ClearEvents(void)
{
  SDL_Event event;

  while(SDL_PollEvent(&event))
    SDL_Delay(50);
}

void doerror(char *fmt, ...)
{
  char strOut[1024];
  va_list marker;

  va_start(marker, fmt);
  vsprintf(strOut, fmt, marker);
  va_end(marker);

  pause(strOut);

  x_gp32_SetCPUSpeed_60();
  exit(0);
}


void LaunchDriver(void)
{
  char command[1024], State[512];
  FILE *fp;
  unsigned long dwHeader, dwFileLength, dwInfoLength;
  int    i, j, k, x;
  int    n_read, n_axf_ro, n_axf_rw;
  unsigned char *p_axf,  *p_axf_ro, *p_axf_rw;
  unsigned char *p_work, *p_temp;
  FILTER *Filter = CurrentFilter();
  if(!Filter) doerror("Launch - CurrentFilter = NULL");
  
  sprintf(command, "%s%s.fxe", 
                   Config.DriverLocations[Config.CurrentDriverLocation].Path,
                   drivers[Filter->__SelectedGameID].fxe);
                   
  if(!Config.OptDriverDisabled)
  {
	  for(x = 0; x < Config.OptDriverCount; x++)
	  {
	  	if(strcasecmp(drivers[Filter->__SelectedGameID].name, Config.OptDrivers[x].Desc) == 0)
	  	{
	  		strcpy(command, Config.OptDrivers[x].Path);
	  		break;
	  	}
	  }                 
  }

  fp = fopen(command, "rb");
  if(!fp)
  {
    pause("Unable to open %s", command);
    return;
  }

  fread(&dwHeader, sizeof(unsigned long), 1, fp);
  fread(&dwFileLength, sizeof(unsigned long), 1, fp);
  fread(&dwInfoLength, sizeof(unsigned long), 1, fp);

  fseek(fp, dwInfoLength + 8, SEEK_SET);

  fread(&n_axf_ro, sizeof(int), 1, fp);
  fread(&n_axf_rw, sizeof(int), 1, fp);

  n_read = n_axf_ro + n_axf_rw;
  p_axf = (unsigned char*) malloc(n_read + 4);
  if (!p_axf)
    doerror("Mem alloc err (%d bytes)", n_read + 4);

  p_axf_ro = p_axf + 4;
  p_axf_rw = p_axf_ro + n_axf_ro;
  *(int*) p_axf = n_read;

  fread(p_axf_rw, n_axf_rw, 1, fp);

  j = n_axf_rw;
  p_work = p_axf_rw;
  for (i = 0; i < j; i ++)
  {
    *p_work ^= 0xff;
    p_work ++;
  }


  fread(p_axf_ro, n_axf_ro, 1, fp);
  fclose(fp);

  j = n_axf_ro;
  k = n_axf_rw;
  p_work = p_axf_ro;
  p_temp = p_axf_rw;
  for (i = 0; i < j; i ++)
  {
    *p_work ^= *p_temp;
    p_work ++;
    p_temp ++;
    if (p_temp >= (p_axf_rw + k))
      p_temp = p_axf_rw;
  }

  sprintf(command, "%s gp:\\gpmm\\mamegp32.fxe %s %s", 
                   command, 
                   drivers[Filter->__SelectedGameID].name,
                   GetFilterState(State) );

  x_gp32_SetCPUSpeed_60();

  SDL_Quit();

#ifndef __WIN32__
  asm volatile(" mov r0, %0 \n"
               " mov r1, %1 \n"
               " swi #0x05 \n"::"r"(p_axf), "r" (command): "r0", "r1");
#endif
  exit(0);
}


char *AppPath(void)
{
#ifdef __WIN32__
  return "";
#else
  return (char*)0x0C7B0C00;
#endif
}

SDL_Rect RECT(int x, int y, int w, int h)
{
	SDL_Rect r;
	r.x = x;
	r.y = y;
	r.h = h;
	r.w = w;
	return(r);
}

