#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <stdarg.h>

#ifndef __WIN32__
#include <x_gp32.h>
#endif

#include "util.h"
#include "video.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "SDL_gfx_fnt.h"

/*#include "SDL_gfx_5x8_fnt.h"*/
/*#include "SDL_gfx_5x7_fnt.h"*/
#include "SDL_gfx_7x13_fnt.h"
#include "SDL_gfx_7x13B_fnt.h"
/*#include "SDL_gfx_7x13O_fnt.h"*/
/*#include "SDL_gfx_9x18_fnt.h"*/
#include "SDL_gfx_9x18B_fnt.h"

void InitializeVideo(void);

int Video_Initialized=0;


SDL_Surface *screen=NULL;
VIDEO_VARIS Video_Varis;

static int BGColor = 255;


void InitializeVideo(void)
{
  Uint32 video_flags = SDL_HWSURFACE | SDL_HWPALETTE | SDL_DOUBLEBUF;
  Uint32 video_bits=0;

#ifndef __WIN32__
  video_flags |= SDL_FULLSCREEN;
#endif

  x_gp32_SetCPUSpeed_60();
  video_bits=8;

  if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK|SDL_INIT_TIMER) < 0 )
  {
    fprintf(stderr, "Couldn't initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }

  screen = SDL_SetVideoMode(320, 240, video_bits, video_flags);
  if ( screen == NULL )
  {
  	fprintf(stderr, "Couldn't set %dx%dx%d video mode: %s\n", 320, 240, video_bits, SDL_GetError());
    exit(1);
  }

  SDL_JoystickEventState(SDL_ENABLE);
  SDL_JoystickOpen(0);
  
  Video_Initialized = 1;
}



void ClearScreen(void)
{
	if(!screen) InitializeVideo();
	
/* 255 white
 * 254 light yellow
 * 245 orange
 * 225 redish
 * 215 very light purple
 * 185 light green-yellow
 * 165 darkish red
 * 155 very light blue
 * 150 light green
 * 110 bluish purple
 * 111 "       "
 * 108 light brownish
 * 85 green
 * 80 darker green
 * 75 darker blue
 * 70 darker purpleish blue
 * 65 real dark purple
 * 55 light blue
 * 50 not a light blue
 * 45 medium blue-green
 * 35 blue
 * 10 nice blue
 * 5 dark blue
 * 0 black
*/	
  SDL_FillRect(screen, NULL, BGColor);
}

void SetFontColor(int r, int g, int b, int i)
{
	Video_Varis.textcolor_r = r;
	Video_Varis.textcolor_g = g;
	Video_Varis.textcolor_b = b;
	Video_Varis.textcolor_i = i;
}


void pause(char *fmt, ...)
{
  char str[4096];
  va_list marker;
  SDL_Event event;

  va_start(marker, fmt);
  vsprintf(str, fmt, marker);
  va_end(marker);

	if(!screen) InitializeVideo();

  SDL_FillRect(screen,NULL,0);

  gfxPrimitivesSetFont(&SDL_gfx_font_7x13B_fnt,7,13);   /* SDL_gfx_font_9x18_fnt */
//  gfxPrimitivesSetFont(&SDL_gfx_font_5x8_fnt,5,8);  
  stringRGBA(screen,10,20, str,255,255,255,255);

  SDL_Flip(screen);

  ClearEvents();
  while (1)
  {
    while ( SDL_PollEvent(&event) )
	  {
	    switch (event.type)
	    {
	      case SDL_JOYBUTTONDOWN:
	      case SDL_KEYDOWN:
	        ClearEvents();
	        return;
	    }
	  }
  }
}

void TextAt(int x, int y, char *fmt, ...)
{
  char str[4096];
  va_list marker;

	if(!screen) InitializeVideo();

  va_start(marker, fmt);
  vsprintf(str, fmt, marker);
  va_end(marker);
  
  stringRGBA(screen, x, y, str, Video_Varis.textcolor_r, 
                                Video_Varis.textcolor_g,
                                Video_Varis.textcolor_b,
                                Video_Varis.textcolor_i); 
  
}

